import os
import time
from github import Github
from django.db import models
from calaccess_raw import get_model_list
from django.template.loader import render_to_string
from calaccess_raw.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    help = 'Create GitHub issues for fields missing verbose and/or help text'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--dry-run",
            action="store_true",
            dest="dry_run",
            default=False,
            help="Print text of issues without sending to Github"
        )

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        """
        Connect to Github using token stored in environment, loop over model fields, and \
        create an issue for any choice field missing 
        """
        self.dry_run = options["dry_run"]
        # set up connect to Github account
        self.gh = Github(os.getenv('GITHUB_TOKEN'))
        self.org = self.gh.get_organization("california-civic-data-coalition")
        self.repo = self.org.get_repo("django-calaccess-raw-data")
        self.labels = [
            self.repo.get_label("small"),
            self.repo.get_label("documentation"),
            self.repo.get_label("enhancement"),
        ]
        self.header(
            "Creating GitHub issues for model choice fields"
        )

        model_list = sorted(
            get_model_list(),
            key=lambda x: (x().klass_group, x().klass_name)
        )

        models_to_fix = []

        for m in model_list:
            fields_to_fix = {}

            for f in m._meta.fields:
                if f.name == 'id':
                    continue
                # test for verbose name
                if not f.__dict__['_verbose_name']:
                    fields_to_fix[f] = {'no_verbose': True, 'no_help': False}
                elif len(f.__dict__['_verbose_name']) == 0:
                    fields_to_fix[f] = {'no_verbose': True, 'no_help': False}
                
                # test for help text
                if len(f.help_text) == 0:
                    try:
                        fields_to_fix[f]['no_help'] = True
                    except KeyError:
                        fields_to_fix[f] = {'no_verbose': False, 'no_help': True}
            
            if len(fields_to_fix) > 0:
                fs = []
                for k, v in fields_to_fix.items():
                    fs.append((k, v))

                models_to_fix.append(
                    (m, tuple(fs))
                )

        for model, fields in models_to_fix:

            context = dict(
                model_name=model.__name__,
                model_docs=model().DOCUMENTCLOUD_PAGES,
                file_name=model.__module__.split('.')[-1] + '.py',
                fields=fields,
            )

            title = "Add verbose and/or help text fields on {model_name} (in \
{file_name})".format(**context)

            body = render_to_string(
                'toolbox/createverboseandhelptextissues.md',
                context,
            )

            self.log("-- Creating issue for {model_name}".format(**context))
            if self.dry_run:
                print '=========================='
                print title
                print '--------------------------'
                print body
                print '=========================='
            else:
                self.repo.create_issue(
                    title,
                    body=body,
                    labels=self.labels,
                )
                time.sleep(2.5)




import os
BASE_DIR = os.path.dirname(__file__)
REPO_DIR = os.path.join(BASE_DIR, os.pardir)
SECRET_KEY = 'w11nbg_3n4+e@qk^b55qgo5qygesn^3=&s1kwtlbpkai$(1jv3'
DEBUG = False
ALLOWED_HOSTS = [
    'localhost',
    '127.0.0.1',
]
STATIC_ROOT = os.path.join(BASE_DIR, ".static")
MEDIA_ROOT = os.path.join(BASE_DIR, ".media")

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'toolbox',
    'calaccess_raw',
)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'urls'
WSGI_APPLICATION = 'wsgi.application'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'calaccess_raw',
        'USER': 'postgres',
        'PASSWORD': '',
        'HOST': 'localhost',
        'PORT': '5432'
    },
}
CALACCESS_STORE_ARCHIVE = False

IA_STORAGE_ACCESS_KEY = os.getenv('IA_STORAGE_ACCESS_KEY')
IA_STORAGE_SECRET_KEY = os.getenv('IA_STORAGE_SECRET_KEY')
IA_STORAGE_COLLECTION = 'test_collection'
IA_STORAGE_CONTRIBUTOR = 'palewire'
IA_STORAGE_CREATOR = "palewire"
IA_STORAGE_PUBLISHER = 'california-civic-data-coalition/django-calaccess-raw-data'
IA_STORAGE_MEDIATYPE = "data"
IA_STORAGE_SUBJECT = ['test']

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = False
USE_L10N = False
USE_TZ = True
STATIC_URL = '/static/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        },
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'logfile': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'django.log'),
            'maxBytes': 1024*1024*5,  # 5MB
            'backupCount': 0,
            'formatter': 'verbose',
        },
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s|%(asctime)s|%(module)s|%(message)s',
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
        'simple': {
            'format': '%(message)s'
        },
    },
    'loggers': {
        'calaccess_raw': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'postgres_copy': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'ia_storage': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    }
}

try:
    from settings_local import *
except ImportError:
    pass



import os
import time
import calculate
from github import Github
from django.conf import settings
from calaccess_raw import get_model_list
from calaccess_raw.management.commands import CalAccessCommand
from django.contrib.humanize.templatetags.humanize import intcomma


class Command(CalAccessCommand):
    help = "Creating GitHub issues for model models without a UNIQUE_KEY"

    def set_options(self, *args, **kwargs):
        """
        Hook up with the GitHub API and prepare to create issues.
        """
        self.gh = Github(os.getenv('GITHUB_TOKEN'))
        self.org = self.gh.get_organization("california-civic-data-coalition")
        self.repo = self.org.get_repo("django-calaccess-raw-data")
        self.labels = [
            self.repo.get_label("small"),
            self.repo.get_label("documentation"),
            self.repo.get_label("enhancement"),
        ]
        self.milestone = self.repo.get_milestone(4)

    def handle(self, *args, **kwargs):
        """
        Make it happen.
        """
        self.set_options()
        self.header(
            "Creating GitHub issues for model models without a UNIQUE_KEY"
        )

        # Loop through all the models and find any fields without docs
        missing_list = []
        model_count = 0
        for m in get_model_list():
            model_count += 1
            if not m.UNIQUE_KEY:
                self.log("Missing: %s.%s" % (
                        m().klass_group,
                        m().klass_name,
                    )
                )
                missing_list.append(m)

        # If everything is done, declare victory
        missing_count = len(missing_list)
        if not missing_count:
            self.success("All %s models have a UNIQUE_KEY!" % missing_count)
            return False

        # If not, loop through the missing and create issues
        self.log(
            "- %s/%s (%d%%) of fields lack a UNIQUE_KEY" % (
                intcomma(missing_count),
                intcomma(model_count),
                calculate.percentage(missing_count, model_count)
            )
        )
        for model in missing_list[1:]:
            self.create_issue(model)

    def create_issue(self, model):
        """
        Create a GitHub issue for the provided model and field.
        """
        title = TITLE_TEMPLATE % model().klass_name

        body = BODY_TEMPLATE % (
               model().klass_name,
               model().klass_group,
               model().klass_group,
        )
        self.log("-- Creating issue for %s" % model().klass_name)
        self.repo.create_issue(
            title,
            body=body,
            labels=self.labels,
            milestone=self.milestone
        )
        time.sleep(2.5)

TITLE_TEMPLATE = """
Add UNIQUE_KEY to the ``%s`` database model
""".replace("\n", "")

BODY_TEMPLATE = """
## Your mission

Add a ``UNIQUE_KEY`` setting to the ``%s`` database model.

## Here's how

**Step 1**: Claim this ticket by leaving a comment below. Tell everyone you're ON IT!

**Step 2**: In a separate tab, open up the <a href="http://www.documentcloud.org/documents/1308002-cal-access-about.html#document/p100">official state documentation</a> and find the page that defines this table. They are listed there in alphabetical order.

![The docs](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367001.08.png)

**Step 3**: Find the "PK_" row in the index list included with the table's definition that spells out the list of "column codes" that make up the unique key for each row. If it lacks that documentation. Note that in the ticket and close it now.

![The definition](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1442811831.94.png)

**Step 4**: Return to this tab.

**Step 5**: Open up the file that contains this model. It should be in <a href="https://github.com/california-civic-data-coalition/django-calaccess-raw-data/blob/master/calaccess_raw/models/%s.py">calaccess_raw.models.%s.py</a>.

**Step 6**: Hit the little pencil button in the upper-right corner of the code box to begin editing the file.

![Edit](https://dl.dropboxusercontent.com/u/3640647/ScreenCloud/1440367320.67.png)

**Step 7**: Find this model in the file. (Clicking into the box and searching with CTRL-F can help you here.) Once you find it, we expect the model to lack a ``UNIQUE_KEY`` attribute indented just under the documentation.

```python
class WhateverYourModel(CalAccessBaseModel):
    '''
    There will be some documentation here but don't touch it.
    '''
    field1 = models.CharField(blah='blah')
```
**Step 8**: Add this model's ``UNIQUE_KEY`` list of column codes, to our field definition by inserting it under the documentation. That should look something like this:

```python
class WhateverYourModel(CalAccessBaseModel):
    '''
    There will be some documentation here but don't touch it.
    '''
    UNIQUE_KEY = ['FIELD1', 'FIELD2', 'FIELD3']
    field1 = models.CharField(blah='blah')
```

**Step 9**: Scroll down below the code box and describe the change you've made in the commit message. Press the button below.

**Step 10**:  Review your changes and create a pull request submitting them to the core team for inclusion.

That's it! Mission accomplished!
"""



"""
Configuration of the Django app.
"""
from django.apps import AppConfig


class CalAccessRawConfig(AppConfig):
    """
    Configuration of the `calaccess_raw` Django app.
    """

    name = "calaccess_raw"
    verbose_name = "CAL-ACCESS raw data"
    default_auto_field = "django.db.models.BigAutoField"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import agate
from django.db.models import Count
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw import get_model_list


class Command(CalAccessCommand):
    help = "Find db values in choice fields not defined in the fields choices attr."

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "-a",
            "--app-name",
            dest="app_name",
            default="calaccess_raw",
            help="Name of Django app with models into which data will "
                 "be imported (if other not calaccess_raw)"
        )

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)

        model_list = sorted(
            get_model_list(),
            key=lambda x: (x().klass_group, x().klass_name)
        )

        results = []

        for m in model_list:
            if self.verbosity > 1:
                self.log(
                    " Verifying {0}.{1} fields".format(
                        m().klass_group,
                        m.__name__,
                    )
                )
            for f in m._meta.fields:
                if f.choices:
                    for value, count in m.objects.order_by().values_list(
                        f.name,
                    ).annotate(Count(f.name)):
                        if (
                            value not in [x[0] for x in f.choices] and
                            value != '' and
                            value is not None
                        ):
                            if self.verbosity > 2:
                                self.failure(
                                    "  Undefined value for {0}: {1} ({2} occurrences)".format(
                                        f.name,
                                        value,
                                        count
                                    )
                                )
                            results.append((
                                m().klass_group,
                                m.__name__,
                                f.name,
                                value,
                                count,
                            ))

        if len(results) > 0:
            self.failure("{} undefined choice field values".format(len(results)))

            table = agate.Table(
                results,
                ['group', 'model', 'field', 'undefined_value', 'occurrences']
            )
            table.print_table(max_rows=None, max_column_width=50)
        else:
            self.success("No undefined choice field values")



"""
Tests annotations of the data.
"""
# Testing
from django.test import TestCase

# Stuff to test
import os
from calaccess_raw import annotations

# Logging
import logging

logger = logging.getLogger(__name__)


class AnnotationTestCase(TestCase):
    """
    Tests annotations of the data.
    """

    multi_db = True

    def test_filingform(self):
        """
        Test attributes of the FilingForm object.
        """
        ff = annotations.get_form("F400")
        ff.get_models()
        ff.__str__()
        s = ff.get_section("P1")
        s.__str__()

    def test_documentcloud(self):
        """
        Test attributes of the DocumentCloud object.
        """
        dc = annotations.DocumentCloud("2753585", 1, 1)
        if os.path.exists(dc.metadata_filename):
            os.remove(dc.metadata_filename)
        dc.metadata
        dc.metadata
        dc.title
        dc.canonical_url
        dc.thumbnail_url
        dc.pdf_url
        dc.text_url
        dc.num_pages
        dc.pages
        dc.formatted_page_nums
        dc2 = annotations.DocumentCloud("2753585")
        dc2.canonical_url
        dc2.num_pages
        dc2.formatted_page_nums
        dc3 = annotations.DocumentCloud("2753585", None, 1)
        dc3.num_pages
        dc3.formatted_page_nums
        dc4 = annotations.DocumentCloud("2753585", 1, None)
        dc4.num_pages
        dc4.formatted_page_nums



"""
Download, unzip, clean and load the latest CAL-ACCESS database ZIP.
"""
# Files
import os

# Commands
from django.core.management import call_command
from calaccess_raw.management.commands import CalAccessCommand

# Models
from calaccess_raw import get_model_list

# Logging
import logging

logger = logging.getLogger(__name__)


class Command(CalAccessCommand):
    """
    Download, unzip, clean and load the latest CAL-ACCESS database ZIP.
    """

    help = "Download, unzip, clean and load the latest CAL-ACCESS database ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep zip, unzipped, TSV and CSV files",
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # set / compute any attributes that multiple class methods need
        self.keep_files = options["keep_files"]

        # Download
        call_command(
            "downloadcalaccessrawdata",
            verbosity=self.verbosity,
        )
        if self.verbosity:
            self.duration()

        # Extract
        call_command("extractcalaccessrawfiles", keep_files=self.keep_files)
        if self.verbosity:
            self.duration()

        # Clean
        self.clean()
        if self.verbosity:
            self.duration()

        # Load
        self.load()
        if self.verbosity:
            self.duration()
            self.success("Done!")

    def clean(self):
        """
        Clean up the raw data files from the state so they are ready to get loaded into the database.
        """
        if self.verbosity:
            self.header("Cleaning data files")

        tsv_list = [f for f in os.listdir(self.tsv_dir) if ".TSV" in f.upper()]

        # Loop through all the files in the source directory
        for name in tsv_list:
            call_command(
                "cleancalaccessrawfile",
                name,
                verbosity=self.verbosity,
                keep_file=self.keep_files,
            )

    def load(self):
        """
        Loads the cleaned up csv files into the database.
        """
        if self.verbosity:
            self.header("Loading data files")

        model_list = [
            x for x in get_model_list() if os.path.exists(x.objects.get_csv_path())
        ]

        for model in model_list:
            call_command(
                "loadcalaccessrawfile",
                model.__name__,
                verbosity=self.verbosity,
                keep_file=self.keep_files,
            )



"""
Clean a source CAL-ACCESS TSV file and reformat it as a CSV.
"""
import os
import csv

import csvkit
from calaccess_raw.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Clean a source CAL-ACCESS TSV file and reformat it as a CSV.
    """

    help = "Clean a source CAL-ACCESS TSV file and reformat it as a CSV"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "file_name",
            help="Name of the TSV file to be cleaned and discarded for a CSV",
        )
        parser.add_argument(
            "--keep-file",
            action="store_true",
            dest="keep_file",
            default=False,
            help="Keep original TSV file",
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Set all the config options
        self.set_options(options)

        # If the file has data ...
        if self.row_count:
            # Walk through the raw TSV file and create a clean CSV file
            if self.verbosity > 1:
                self.log(" Cleaning %s" % self.file_name)
            self.clean()

        # Unless keeping files, remove the raw TSV file
        if not options["keep_file"]:
            os.remove(self.tsv_path)

    def set_options(self, options):
        """
        Set options for use in other methods.
        """
        # Set options
        self.file_name = options["file_name"]

        # Set log variables
        self.log_dir = os.path.join(self.data_dir, "log/")
        self.log_name = self.file_name.lower().replace("tsv", "errors.csv")
        self.error_log_path = os.path.join(self.log_dir, self.log_name)
        self.log_rows = []

        # Make sure the log directory exists
        os.path.exists(self.log_dir) or os.makedirs(self.log_dir)

        # Input and output paths
        self.tsv_path = os.path.join(self.tsv_dir, self.file_name)
        self.csv_name = self.file_name.lower().replace("tsv", "csv")
        self.csv_path = os.path.join(self.csv_dir, self.csv_name)

        # Pull and clean the headers
        self.headers = self.get_headers()
        self.headers_count = len(self.headers)

        # Get the row count
        with open(self.tsv_path, "r") as tsv_file:
            self.row_count = max(sum(1 for line in tsv_file), 0)

    def get_headers(self):
        """
        Returns the headers from the TSV file.
        """
        with open(self.tsv_path, "r") as tsv_file:
            tsv_reader = csvkit.reader(tsv_file, delimiter=str("\t"))
            try:
                return next(tsv_reader)
            except StopIteration:
                return []

    def _convert_tsv(self):
        """
        Given it a raw list of rows from a TSV, yields cleaned rows for a CSV.
        """
        with open(self.tsv_path, "rb") as tsv_file:
            # Pop the headers out of the TSV file
            next(tsv_file)

            # Loop through all the rows
            for tsv_line in tsv_file:
                # Decode the line for testing
                tsv_line = tsv_line.decode("ascii", "replace")

                # If the line is empty skip it
                if not tsv_line.strip():
                    continue

                # Nuke any null bytes
                if tsv_line.count("\x00"):
                    tsv_line = tsv_line.replace("\x00", " ")

                # Nuke the ASCII "substitute character." chr(26) in Python
                if tsv_line.count("\x1a"):
                    tsv_line = tsv_line.replace("\x1a", "")

                # Remove any extra newline chars
                tsv_line = (
                    tsv_line.replace("\r\n", "").replace("\r", "").replace("\n", "")
                )

                # Split on tabs so we can later spit it back out as a CSV row
                csv_line = tsv_line.split("\t")
                csv_field_count = len(csv_line)

                # If it matches the header count, yield it
                if csv_field_count == self.headers_count:
                    yield csv_line
                else:
                    # Otherwise log it
                    self.log_rows.append(
                        [self.headers_count, csv_field_count, ",".join(csv_line)]
                    )

    def clean(self):
        """
        Cleans the provided source TSV file and writes it out in CSV format.
        """
        # Create the output object
        with open(self.csv_path, "w") as csv_file:
            # Create the CSV writer
            csv_writer = csvkit.writer(csv_file)
            # Write the headers
            csv_writer.writerow(self.headers)
            # Write out the rows
            [csv_writer.writerow(row) for row in self._convert_tsv()]

        # Log errors if there are any
        if self.log_rows:
            # Log to the terminal
            if self.verbosity > 2:
                msg = "  {} errors logged (not including empty lines)"
                self.failure(msg.format(len(self.log_rows)))

            # Log to the file
            with open(self.error_log_path, "w") as log_file:
                log_writer = csvkit.writer(log_file, quoting=csv.QUOTE_ALL)
                log_writer.writerow(["headers", "fields", "value"])
                log_writer.writerows(self.log_rows)



# Generated by Django 3.2.4 on 2021-06-14 15:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0019_alter_rawdataversion_download_zip_archive"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="rawdatafile",
            name="download_file_archive",
        ),
        migrations.AddField(
            model_name="rawdatafile",
            name="InternetArchiveFileField",
            field=models.FileField(
                blank=True,
                help_text="An archive of the original raw data file downloaded from CAL-ACCESS.",
                max_length=255,
                upload_to="",
                verbose_name="archive of download file",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-26 16:41
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0008_auto_20161025_1527"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cvr2campaigndisclosurecd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvr2lobbydisclosurecd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvr2registrationcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvr2socd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvr3verificationinfocd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvrcampaigndisclosurecd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvrlobbydisclosurecd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvrregistrationcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                blank=True,
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                null=True,
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="cvrsocd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing id",
            ),
        ),
        migrations.AlterField(
            model_name="debtcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number of the parent filing",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="f495p2cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="f501502cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="f690p2cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lattcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lccmcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lempcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lexpcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="loancd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lobbyamendmentscd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmemployer1cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmemployer2cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lothcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="lpaycd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="rcptcd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="s401cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="s496cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="s497cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
        migrations.AlterField(
            model_name="s498cd",
            name="filing_id",
            field=calaccess_raw.fields.IntegerField(
                db_column="FILING_ID",
                db_index=True,
                help_text="Unique filing identification number",
                verbose_name="filing ID",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-11-23 22:17
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0011_auto_20161028_0129"),
    ]

    operations = [
        migrations.AlterField(
            model_name="expncd",
            name="memo_code",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="MEMO_CODE",
                help_text="Memo Amount? (Date/Amount are informational only). For Form 460 filings, this indicates the record is a sub-item and its amount is included in another item reported on the filing.",
                max_length=1,
            ),
        ),
        migrations.AlterField(
            model_name="loancd",
            name="loan_date1",
            field=calaccess_raw.fields.DateField(
                db_column="LOAN_DATE1",
                help_text="Date the loan was made or received. The content of this column varies based on the schedule/part that the record applies to. See the CAL document for a description of the value.",
                null=True,
            ),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Tests how thoroughly the CAL-ACCESS database has been documented.
"""
# Testing
from django.test import TestCase

# Stuff to test
from calaccess_raw import get_model_list
from calaccess_raw.fields import ForeignKeyField

# Text
import re
import agate

# Logging
import logging

logger = logging.getLogger(__name__)


class DocumentationTestCase(TestCase):
    """
    Tests the documentation of models, fields and values.
    """

    @classmethod
    def setUpClass(cls):
        """
        Load data into the database before running other tests.
        """
        super(DocumentationTestCase, cls).setUpClass()

    def attr_test_output(self, obj_type, attr_name, results):
        """
        Output test results to the console.
        """
        # Load the data
        table = agate.Table(results, ["group", obj_type, attr_name])

        # Count the number with no __str__
        fails = table.where(lambda row: row[attr_name] is False)
        if fails.rows:
            print(
                "Fail: %s %ss are missing %s" % (len(fails.rows), obj_type, attr_name)
            )
            fails.select(["group", obj_type]).print_table(
                max_rows=None,
                max_column_width=50,
            )

    def test_model_str(self):
        """
        Verify that __str__ methods exist and work for all models.
        """
        results = []
        for m in get_model_list():
            # Test that works
            m().__str__()
            # Make sure that it is customized and not the Django defalult
            is_custom = m().__str__() != "%s object" % m.__name__
            results.append([m().klass_group, m.__name__, is_custom])
        self.attr_test_output("model", "__str__", results)

    def test_model_doc(self):
        """
        Verify that __doc__ methods exist and work for all models.
        """
        results = []
        for m in get_model_list():
            if m().__doc__.startswith(m.__name__):
                exists = False
            else:
                exists = True
            results.append([m().klass_group, m.__name__, exists])
        self.attr_test_output("model", "__doc__", results)

    def test_model_unique_key(self):
        """
        Verify that each model has a UNIQUE_KEY attribute set.
        """
        results = []
        # Loop through the models
        for m in get_model_list():

            # Verify the model has a UNIQUE_KEY attribute
            exists = m().UNIQUE_KEY is not None

            # Verify that the field names are not lowercase
            lowercases = (
                sum([1 for k in m().get_unique_key_list() if re.search("[a-z]{1,}", k)])
                or 0
            )
            if lowercases:
                exists = False

            # Verify that the fields actually exist
            missing = [
                f
                for f in m().get_unique_key_list()
                if not hasattr(m(), f.lower()) and not hasattr(m(), f.lower() + "_id")
            ]
            if missing:
                exists = False

            # Pass out results
            results.append([m().klass_group, m.__name__, exists])

        # Print results
        self.attr_test_output("model", "UNIQUE_KEY", results)

    def test_model_documentcloud_pages(self):
        """
        Verify that each model has DOCUMENTCLOUD_PAGES defined.
        """
        results = []
        for m in get_model_list():
            if m().DOCUMENTCLOUD_PAGES:
                exists = True
            else:
                exists = False
            results.append([m().klass_group, m.__name__, exists])
        self.attr_test_output("model", "DOCUMENTCLOUD_PAGES", results)

    def test_model_documentcloud_pages_for_both_links(self):
        """
        Verify that models linked to DocumentCloud reference.
        """
        results = []
        exceptions = [
            # campaign models
            "F501502Cd",
            # "common" models
            "FilernameCd",
            "FilerFilingsCd",
            "FilingsCd",
            "HeaderCd",
            "CvrE530Cd",
            # lobbying models
            "LobbyingChgLogCd",
            "LobbyistContributions1Cd",
            "LobbyistContributions2Cd",
            "LobbyistContributions3Cd",
            "LobbyistEmployer1Cd",
            "LobbyistEmployer2Cd",
            "LobbyistEmployer3Cd",
            "LobbyistEmployerHistoryCd",
            "LobbyistEmployerFirms1Cd",
            "LobbyistEmployerFirms2Cd",
            "LobbyistEmpLobbyist1Cd",
            "LobbyistEmpLobbyist2Cd",
            "LobbyistFirm1Cd",
            "LobbyistFirm2Cd",
            "LobbyistFirm3Cd",
            "LobbyistFirmEmployer1Cd",
            "LobbyistFirmEmployer2Cd",
            "LobbyistFirmLobbyist1Cd",
            "LobbyistFirmLobbyist2Cd",
            "LobbyistFirmHistoryCd",
            # "other" models (this is indeed all of them)
            "AcronymsCd",
            "AddressCd",
            "BallotMeasuresCd",
            "EfsFilingLogCd",
            "FilersCd",
            "FilerAcronymsCd",
            "FilerAddressCd",
            "FilerEthicsClassCd",
            "FilerInterestsCd",
            "FilerLinksCd",
            "FilerStatusTypesCd",
            "FilerToFilerTypeCd",
            "FilerTypesCd",
            "FilerXrefCd",
            "FilerTypePeriodsCd",
            "FilingPeriodCd",
            "GroupTypesCd",
            "ImageLinksCd",
            "LegislativeSessionsCd",
            "LookupCodesCd",
            "NamesCd",
            "ReceivedFilingsCd",
            "ReportsCd",
        ]
        model_list = [x for x in get_model_list() if x.__name__ not in exceptions]

        for m in model_list:

            if m().DOCUMENTCLOUD_PAGES:
                cal_access_tables = False
                map_cal_format = False

                # cycle through objs, checking for IDs
                for doc_pages in m().DOCUMENTCLOUD_PAGES:

                    if doc_pages.id == "2711614-CalAccessTablesWeb":
                        cal_access_tables = True
                    elif doc_pages.id == "2711616-MapCalFormat2Fields":
                        map_cal_format = True

                # if both exist, return True, if both don't exist return false
                if cal_access_tables and map_cal_format:
                    complete = True
                else:
                    complete = False

            # if the object doesn't exist at all, will fail test
            else:
                complete = False

            results.append([m().klass_group, m.__name__, complete])
        self.attr_test_output("model", "DOCUMENTCLOUD_PAGES_COMPLETE", results)

    def test_filing_forms(self):
        """
        Verify that model's with a form field are linked to a FilingForm object.
        """
        results = []
        exceptions = ["HeaderCd"]
        model_list = [x for x in get_model_list() if x.__name__ not in exceptions]
        for m in model_list:
            field_names = m._meta.get_fields()
            if "form_type" in field_names or "form_id" in field_names:
                if m().FILING_FORMS:
                    exists = True
                else:
                    exists = False
                results.append([m().klass_group, m.__name__, exists])
        self.attr_test_output("model", "FILING_FORMS", results)

    def test_field_help_text(self):
        """
        Verify that all fields have help_text documentation.
        """
        results = []
        for m in get_model_list():
            for f in m().get_field_list():
                if f.name == "id":
                    continue
                if len(f.help_text) > 0:
                    exists = True
                else:
                    exists = False
                results.append(
                    [m().klass_group, "%s.%s" % (m.__name__, f.name), exists]
                )
        self.attr_test_output("field", "help_text", results)

    def test_choices(self):
        """
        Verify that valid choices are available for all expected fields on all models.
        """
        # substrings that appear in choice fields
        choice_field_strs = [
            "_cd",
            "_code",
            "_type",
            "status",
            "_lvl",
            "reportname",
            "form_id",
        ]
        exceptions = [
            "LookupCodesCd.code_type",
            "S497Cd.sup_off_cd",
            "FilerStatusTypesCd.status_type",
            "FilerStatusTypesCd.status_desc",
            "FilerTypesCd.filer_type",
        ]

        results = []
        model_list = sorted(
            get_model_list(), key=lambda x: (x().klass_group, x().klass_name)
        )

        for m in model_list:
            for f in m._meta.fields:
                if (
                    any(x in f.name for x in choice_field_strs)
                    and f.name != "memo_code"
                    and f.__class__ is not ForeignKeyField
                    and "{}.{}".format(m().klass_name, f.name) not in exceptions
                ):
                    if not f.choices:
                        results.append(
                            (
                                m().klass_group,
                                m.__name__,
                                f.name,
                                "Has no CHOICES defined",
                            )
                        )

                    if not f.documentcloud_pages:
                        results.append(
                            (
                                m().klass_group,
                                m.__name__,
                                f.name,
                                "Has no `documentcloud_pages` defined",
                            )
                        )

                    # Pull out all the choices in that field
                    for slug, name in f.flatchoices:
                        # Make sure that each has a definition
                        if not name or name == "":
                            results.append(
                                (
                                    m().klass_group,
                                    m.__name__,
                                    f.name,
                                    "Value '%s' undefined in CHOICES" % slug,
                                )
                            )

        table = agate.Table(results, ["group", "model", "field", "message"])
        table.print_table(max_rows=None, max_column_width=50)



# Generated by Django 3.2.13 on 2022-07-25 17:11

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0024_auto_20210822_2158"),
    ]

    operations = [
        migrations.AlterField(
            model_name="namescd",
            name="fullname",
            field=calaccess_raw.fields.CharField(
                db_column="FULLNAME", help_text="Full name", max_length=1000
            ),
        ),
        migrations.AlterField(
            model_name="namescd",
            name="naml",
            field=calaccess_raw.fields.CharField(
                db_column="NAML", help_text="Last name", max_length=1000
            ),
        ),
        migrations.AlterField(
            model_name="namescd",
            name="naml_search",
            field=calaccess_raw.fields.CharField(
                db_column="NAML_SEARCH", help_text="Last name", max_length=1000
            ),
        ),
    ]



import os
import time
from github import Github
from django.db import models
from calaccess_raw import get_model_list
from django.template.loader import render_to_string
from calaccess_raw.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    help = 'Create GitHub issues for model choice fields'

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--dry-run",
            action="store_true",
            dest="dry_run",
            default=False,
            help="Print text of issues without sending to Github"
        )

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        """
        Connect to Github using token stored in environment, loop over model fields, and \
        create an issue for any choice field missing 
        """
        self.dry_run = options["dry_run"]
        # set up connect to Github account
        self.gh = Github(os.getenv('GITHUB_TOKEN'))
        self.org = self.gh.get_organization("california-civic-data-coalition")
        self.repo = self.org.get_repo("django-calaccess-raw-data")
        self.labels = [
            self.repo.get_label("small"),
            self.repo.get_label("documentation"),
            self.repo.get_label("enhancement"),
        ]
        self.header(
            "Creating GitHub issues for model choice fields"
        )

        choice_field_strs = [
            '_cd',
            '_code',
            'type',
            'status',
            '_lvl',
            'reportname',
            'form_id',
        ]
        excluded_fields = [
            'LookupCodesCd.code_type',
            'S497Cd.sup_off_cd',
        ]

        model_list = sorted(
            get_model_list(),
            key=lambda x: (x().klass_group, x().klass_name)
        )

        for m in model_list:
            for f in m._meta.fields:
                if (
                    any(x in f.name for x in choice_field_strs) and
                    f.name != 'memo_code' and
                    f.__class__ is not models.ForeignKey and
                    '{}.{}'.format(m().klass_name, f.name) not in excluded_fields
                ):
                    # make an issue for every choice field missing docs
                    # includes those that are also missing choices
                    if not f.documentcloud_pages:
                        self.create_issue(f)

    def create_issue(self, field):
        """
        Create a GitHub issue for the provided field.
        """
        context = dict(
            field=field,
            model_name=field.model.__name__,
            field_class=field.__class__.__name__,
            db_table=field.model._meta.fields[1].model._meta.db_table,
            has_choices=bool(field.choices),
            has_docs=bool(field.documentcloud_pages),
            file_name=field.model.__module__.split('.')[-1] + '.py',
        )
        title = " to {model}.{field} (in {file_name})".format(
            model=field.model.__name__,
            field=field.name,
            file_name=context['file_name']
        )

        if not field.choices and not field.documentcloud_pages:
            title = 'Add choices and documentcloud_pages' + title
        elif not field.documentcloud_pages:
            title = 'Add documentcloud_pages' + title

        body = render_to_string(
            'toolbox/createchoicefieldissue.md',
            context,
        )

        self.log("-- Creating issue for %s.%s" % (
                field.model.__name__,
                field.name
            )
        )
        if self.dry_run:
            print '=========================='
            print title
            print '--------------------------'
            print body
            print '=========================='
        else:
            self.repo.create_issue(
                title,
                body=body,
                labels=self.labels,
            )
            time.sleep(2.5)



import os
import time
import calculate
from github import Github
from datetime import datetime
from django.conf import settings
from calaccess_raw import get_model_list
from calaccess_raw.management.commands import CalAccessCommand
from django.contrib.humanize.templatetags.humanize import intcomma


class Command(CalAccessCommand):
    help = 'Analyze GitHub commits during one of our code sprints'

    def set_options(self, *args, **kwargs):
        """
        Hook up with the GitHub API.
        """
        self.gh = Github(os.getenv('GITHUB_TOKEN'))
        self.org = self.gh.get_organization("california-civic-data-coalition")
        self.repo_list = [
            self.org.get_repo("django-calaccess-raw-data"),
        ]
        self.start = datetime(2016, 3, 9, 0, 0, 0)
        self.end = datetime(2016, 3, 15, 0, 0, 0)

    def handle(self, *args, **kwargs):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **kwargs)
        self.set_options()
        self.header("Analyzing Code Rush commits")

        for repo in self.repo_list:
            self.log(repo.name)
            commit_count = 0
            addition_count = 0
            author_list = []
            for commit in repo.get_commits(since=self.start):
                commit_count += 1
                #addition_count += commit.stats.additions
                if commit.author not in author_list:
                    author_list.append(commit.author)
                time.sleep(0.1)
            self.log("- Authors: %s" % len(author_list))
            self.log("- Commits: %s" % commit_count)
            #self.log("- Additions: %s" % addition_count)



# Generated by Django 3.2 on 2021-04-22 00:47

import calaccess_raw.annotations.documents
import calaccess_raw.annotations.forms
import calaccess_raw.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0016_auto_20200818_0542"),
    ]

    operations = [
        migrations.AlterField(
            model_name="acronymscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="addresscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="ballotmeasurescd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvr2campaigndisclosurecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvr2lobbydisclosurecd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=57
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=71
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="cvr2lobbydisclosurecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvr2registrationcd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=72
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=87
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=10,
            ),
        ),
        migrations.AlterField(
            model_name="cvr2registrationcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvr2socd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 400 (Statement of Organization (Slate Mailer Organization)): Part 3, Individuals Who Authorize Contents Of Slate Mailers",
                    ),
                    (
                        "",
                        "Form 410 (Statement of Organization (Recipient Committee)): Part 4, Type of Committee",
                    ),
                ],
                db_column="FORM_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711616-MapCalFormat2Fields", start_page=38
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=46, id="2712033-Cal-Format-1-05-02", start_page=45
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=59, id="2712034-Cal-Format-201", start_page=58
                    ),
                ],
                help_text="Form type of the filing the record is included in. This must equal the form_type of the parent filing's cover (CVR) record.",
                max_length=4,
                verbose_name="form type",
            ),
        ),
        migrations.AlterField(
            model_name="cvr2socd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvr3verificationinfocd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvrcampaigndisclosurecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvre530cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[("", "Electronic Form 530: Electronic Issue Advocacy Report")],
                db_column="FORM_TYPE",
                db_index=True,
                help_text="Name of the source filing form or schedule",
                max_length=4,
                verbose_name="form type",
            ),
        ),
        migrations.AlterField(
            model_name="cvre530cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvrf470cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 470: Campaign Disclosure Statement, Short Form (Officeholders and Candidates)",
                    )
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=22
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=29
                    ),
                ],
                help_text="Type of Filing or Formset. The value of this column will always be equal to F470.",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="cvrf470cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvrlobbydisclosurecd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=53
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=66
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="cvrlobbydisclosurecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvrregistrationcd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    ("", "Form 604: Lobbyist Certification Statement"),
                    ("", "Form 606: Notice of Termination"),
                    ("", "Form 607: Notice of Withdrawal"),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=68
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=82
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="cvrregistrationcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="cvrsocd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="debtcd",
            name="expn_code",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[
                    ("CMP", "campaign paraphernalia/miscellaneous"),
                    ("CNS", "campaign consultants"),
                    ("CTB", "contribution (if nonmonetary, explain)*"),
                    ("CVC", "civic donations"),
                    ("FIL", "candidate filing/ballot feeds"),
                    ("FND", "fundraising events"),
                    ("IKD", "In-kind contribution (nonmonetary)"),
                    (
                        "IND",
                        "independent expenditure supporting/opposing others (explain)*",
                    ),
                    ("LEG", "legal defense"),
                    ("LIT", "campaign literature and mailings"),
                    ("LON", "loan"),
                    ("MBR", "member communications"),
                    ("MON", "monetary contribution"),
                    ("MTG", "meetings and appearances"),
                    ("OFC", "office expenses"),
                    ("PET", "petition circulating"),
                    ("PHO", "phone banks"),
                    ("POL", "polling and survey research"),
                    ("POS", "postage, delivery and messenger services"),
                    ("PRO", "professional services (legal, accounting)"),
                    ("PRT", "print ads"),
                    ("RAD", "radio airtime and production costs"),
                    ("RFD", "returned contributions"),
                    ("SAL", "campaign workers salaries"),
                    ("TEL", "T.V. or cable airtime and production costs"),
                    ("TRC", "candidate travel, lodging and meals (explain)"),
                    ("TRS", "staff/spouse travel, lodging and meals (explain)"),
                    (
                        "TSF",
                        "transfer between committees of the same candidate/sponsor",
                    ),
                    ("VOT", "voter registration"),
                    ("WEB", "information technology costs (internet, e-mail)"),
                    ("Fnd", "fundraising events"),
                    ("ofc", "office expenses"),
                    ("'CN", "campaign consultants"),
                    ("*", "Unknown"),
                    ("AIR", "Unknown"),
                    ("BUS", "Unknown"),
                    ("C", "Unknown"),
                    ("CAM", "Unknown"),
                    ("CC", "Unknown"),
                    ("COM", "Unknown"),
                    ("CON", "Unknown"),
                    ("CSN", "Unknown"),
                    ("DEP", "Unknown"),
                    ("EVE", "Unknown"),
                    ("F", "Unknown"),
                    ("FED", "Unknown"),
                    ("fns", "Unknown"),
                    ("G", "Unknown"),
                    ("GGG", "Unknown"),
                    ("HOT", "Unknown"),
                    ("L", "Unknown"),
                    ("LDF", "Unknown"),
                    ("MEE", "Unknown"),
                    ("N", "Unknown"),
                    ("O", "Unknown"),
                    ("OTH", "Unknown"),
                    ("P", "Unknown"),
                    ("PEN", "Unknown"),
                    ("S", "Unknown"),
                    ("SPE", "Unknown"),
                    ("STA", "Unknown"),
                    ("T", "Unknown"),
                    ("TAX", "Unknown"),
                    ("TRA", "Unknown"),
                    ("V", "Unknown"),
                    ("X", "Unknown"),
                ],
                db_column="EXPN_CODE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=11
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=14, id="2712034-Cal-Format-201", start_page=13
                    ),
                ],
                help_text="Expense Code",
                max_length=4,
                verbose_name="expense code",
            ),
        ),
        migrations.AlterField(
            model_name="debtcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="efsfilinglogcd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 400: Statement of Organization (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 401: Campaign Disclosure Statement (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 402: Statement of Termination (Slate Mailer Organization)",
                    ),
                    ("", "Form 410: Statement of Organization (Recipient Committee)"),
                    (
                        "",
                        "Form 425: Semi-Annual Statement of No Activity (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("", "Form 465: Supplemental Independent Expenditure Report"),
                    ("", "Form 496: Late Independent Expenditure Report"),
                    ("", "Form 497: Late Contribution Report"),
                    ("", "Form 498: Late Payment Report (Slate Mailer Organization)"),
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    ("", "Form 604: Lobbyist Certification Statement"),
                    ("", "Form 606: Notice of Termination"),
                    ("", "Form 607: Notice of Withdrawal"),
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                    ("BADFORMAT 253", "Unknown"),
                    ("form", "Unknown"),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=8, id="2711624-Overview", start_page=4
                    )
                ],
                help_text="Name of the source filing form or schedule",
                max_length=250,
                verbose_name="form type",
            ),
        ),
        migrations.AlterField(
            model_name="efsfilinglogcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="expn_code",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[
                    ("CMP", "campaign paraphernalia/miscellaneous"),
                    ("CNS", "campaign consultants"),
                    ("CTB", "contribution (if nonmonetary, explain)*"),
                    ("CVC", "civic donations"),
                    ("FIL", "candidate filing/ballot feeds"),
                    ("FND", "fundraising events"),
                    ("IKD", "In-kind contribution (nonmonetary)"),
                    (
                        "IND",
                        "independent expenditure supporting/opposing others (explain)*",
                    ),
                    ("LEG", "legal defense"),
                    ("LIT", "campaign literature and mailings"),
                    ("LON", "loan"),
                    ("MBR", "member communications"),
                    ("MON", "monetary contribution"),
                    ("MTG", "meetings and appearances"),
                    ("OFC", "office expenses"),
                    ("PET", "petition circulating"),
                    ("PHO", "phone banks"),
                    ("POL", "polling and survey research"),
                    ("POS", "postage, delivery and messenger services"),
                    ("PRO", "professional services (legal, accounting)"),
                    ("PRT", "print ads"),
                    ("RAD", "radio airtime and production costs"),
                    ("RFD", "returned contributions"),
                    ("SAL", "campaign workers salaries"),
                    ("TEL", "T.V. or cable airtime and production costs"),
                    ("TRC", "candidate travel, lodging and meals (explain)"),
                    ("TRS", "staff/spouse travel, lodging and meals (explain)"),
                    (
                        "TSF",
                        "transfer between committees of the same candidate/sponsor",
                    ),
                    ("VOT", "voter registration"),
                    ("WEB", "information technology costs (internet, e-mail)"),
                    ("ctb", "contribution (if nonmonetary, explain)*"),
                    ("ikd", "In-kind contribution (nonmonetary)"),
                    ("Mon", "monetary contribution"),
                    ("ofc", "office expenses"),
                    ("OFc", "office expenses"),
                    ("Ofc", "office expenses"),
                    ("", "Unknown"),
                    ("*", "Unknown"),
                    ("0", "Unknown"),
                    ("001", "Unknown"),
                    ("011", "Unknown"),
                    ("200", "Unknown"),
                    ("401", "Unknown"),
                    ("ADV", "Unknown"),
                    ("ANN", "Unknown"),
                    ("APR", "Unknown"),
                    ("AUG", "Unknown"),
                    ("AUT", "Unknown"),
                    ("Ban", "Unknown"),
                    ("BAN", "Unknown"),
                    ("BOO", "Unknown"),
                    ("BOX", "Unknown"),
                    ("C", "Unknown"),
                    ("CAT", "Unknown"),
                    ("CC", "Unknown"),
                    ("CHE", "Unknown"),
                    ("CIV", "Unknown"),
                    ("CNT", "Unknown"),
                    ("CON", "Unknown"),
                    ("COP", "Unknown"),
                    ("CRE", "Unknown"),
                    ("CSN", "Unknown"),
                    ("CT", "Unknown"),
                    (",CT", "Unknown"),
                    (".CT", "Unknown"),
                    ("CTN", "Unknown"),
                    ("CVD", "Unknown"),
                    ("DAT", "Unknown"),
                    ("DEC", "Unknown"),
                    ("Dem", "Unknown"),
                    ("DIN", "Unknown"),
                    ("Don", "Unknown"),
                    ("DON", "Unknown"),
                    ("Ear", "Unknown"),
                    ("EIM", "Unknown"),
                    ("EMP", "Unknown"),
                    ("F", "Unknown"),
                    ("FAX", "Unknown"),
                    ("FDN", "Unknown"),
                    ("FED", "Unknown"),
                    ("FEE", "Unknown"),
                    ("FIN", "Unknown"),
                    ("Fun", "Unknown"),
                    ("FUN", "Unknown"),
                    ("G", "Unknown"),
                    ("GEN", "Unknown"),
                    ("GGG", "Unknown"),
                    ("GOT", "Unknown"),
                    ("IEs", "Unknown"),
                    ("IN-", "Unknown"),
                    ("Ina", "Unknown"),
                    ("INK", "Unknown"),
                    ("INS", "Unknown"),
                    ("ITE", "Unknown"),
                    ("JAN", "Unknown"),
                    ("JUL", "Unknown"),
                    ("JUN", "Unknown"),
                    ("KIC", "Unknown"),
                    ("L", "Unknown"),
                    ("LEV", "Unknown"),
                    ("Lit", "Unknown"),
                    ("LN#", "Unknown"),
                    ("LOG", "Unknown"),
                    ("M", "Unknown"),
                    ("MAI", "Unknown"),
                    ("Mar", "Unknown"),
                    ("MAR", "Unknown"),
                    ("MAY", "Unknown"),
                    ("MED", "Unknown"),
                    ("MEE", "Unknown"),
                    ("MGT", "Unknown"),
                    ("Mis", "Unknown"),
                    ("MRB", "Unknown"),
                    ("NGP", "Unknown"),
                    ("NON", "Unknown"),
                    ("NOT", "Unknown"),
                    ("NOV", "Unknown"),
                    ("O", "Unknown"),
                    ("OCT", "Unknown"),
                    (".OF", "Unknown"),
                    ("OFF", "Unknown"),
                    ("OPE", "Unknown"),
                    ("OTH", "Unknown"),
                    ("P", "Unknown"),
                    ("Pac", "Unknown"),
                    ("PAI", "Unknown"),
                    ("PAR", "Unknown"),
                    ("PAY", "Unknown"),
                    ("PEN", "Unknown"),
                    ("PMT", "Unknown"),
                    (".PO", "Unknown"),
                    ("Pos", "Unknown"),
                    ("PRE", "Unknown"),
                    ("PRI", "Unknown"),
                    ("PRP", "Unknown"),
                    ("R", "Unknown"),
                    (".Re", "Unknown"),
                    (".RE", "Unknown"),
                    ("REF", "Unknown"),
                    ("REI", "Unknown"),
                    ("RFP", "Unknown"),
                    ("S", "Unknown"),
                    ("S-A", "Unknown"),
                    ("SA", "Unknown"),
                    ("Sal", "Unknown"),
                    ("S C", "Unknown"),
                    ("S.C", "Unknown"),
                    ("SCU", "Unknown"),
                    ("SEE", "Unknown"),
                    ("SEN", "Unknown"),
                    ("SEP", "Unknown"),
                    ("S.M.", "Unknown"),
                    ("SOF", "Unknown"),
                    ("SWI", "Unknown"),
                    ("T", "Unknown"),
                    ("TAX", "Unknown"),
                    ("TB", "Unknown"),
                    ("TB,", "Unknown"),
                    ("TIC", "Unknown"),
                    ("Tor", "Unknown"),
                    ("TRA", "Unknown"),
                    ("TRF", "Unknown"),
                    ("TRV", "Unknown"),
                    ("UN", "Unknown"),
                    ("UTI", "Unknown"),
                    ("V", "Unknown"),
                    ("VEN", "Unknown"),
                    ("-VO", "Unknown"),
                    ("VOI", "Unknown"),
                    ("VOY", "Unknown"),
                    ("WI", "Unknown"),
                    ("x", "Unknown"),
                    ("X", "Unknown"),
                    ("S-6", "Unknown"),
                    ("S.M", "Unknown"),
                    ("S-4", "Unknown"),
                    ("SA:", "Unknown"),
                    ("100", "Unknown"),
                    ("RFN", "Unknown"),
                    ("REN", "Unknown"),
                    ("003", "Unknown"),
                    ("S-1", "Unknown"),
                    ("08", "Unknown"),
                ],
                db_column="EXPN_CODE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=11
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=14, id="2712034-Cal-Format-201", start_page=13
                    ),
                ],
                help_text="The type of expenditure",
                max_length=4,
                verbose_name="expense code",
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "F450P5",
                        "Form 450 (Campaign Disclosure Statement, Short Form (Recipient Committee)): Part 5, Payments Made",
                    ),
                    (
                        "D",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule D, Summary of Expenditures Supporting / Opposing Other Candidates, Measures and Committees",
                    ),
                    (
                        "E",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule E, Payments Made",
                    ),
                    (
                        "G",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule G, Payments Made by an Agent or Independent Contractor (on Behalf of This Committee)",
                    ),
                    (
                        "F461P5",
                        "Form 461 (Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)): Part 5, Contributions (Including Loans, Forgiveness of Loans, and LoanGuarantees) and Expenditures Made",
                    ),
                    (
                        "F465P3",
                        "Form 465 (Supplemental Independent Expenditure Report): Part 3, Independent Expenditures Made",
                    ),
                    (
                        "",
                        "Form 900: Campaign Disclosure Statement (Public employee retirement board candidate)",
                    ),
                ],
                db_column="FORM_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=31
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=42
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=6,
            ),
        ),
        migrations.AlterField(
            model_name="expncd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="f495p2cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 495: Supplemental Pre-Election Campaign Statement (Recipient Committee)",
                    ),
                ],
                db_column="FORM_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=26
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=35
                    ),
                ],
                help_text="Name of the source filing form to which the Form 495 is attached (must equal Form_Type in CVR record)",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="f495p2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="f501502cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 501: Candidate Intention Statement"),
                    ("", "Form 502: Campaign Bank Account Statement"),
                ],
                db_column="FORM_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711614-CalAccessTablesWeb", start_page=58
                    )
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="f501502cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="f690p2cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=58
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=72
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="f690p2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="fileracronymscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="fileraddresscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerethicsclasscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerfilingscd",
            name="form_id",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 400: Statement of Organization (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 401: Campaign Disclosure Statement (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 402: Statement of Termination (Slate Mailer Organization)",
                    ),
                    ("", "Form 405: Amendment to Campaign Disclosure Statement"),
                    ("", "Form 410: Statement of Organization (Recipient Committee)"),
                    ("", "Form 415: Title Unknown"),
                    ("", "Form 416: Title Unknown"),
                    (
                        "",
                        "Form 419: Campaign Disclosure Statement, Long Form (Ballot Measure Committee)",
                    ),
                    (
                        "",
                        "Form 420: Campaign Disclosure Statement, Long Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 425: Semi-Annual Statement of No Activity (Recipient Committee)",
                    ),
                    ("", "Form 430: Title Unknown"),
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("", "Form 465: Supplemental Independent Expenditure Report"),
                    (
                        "",
                        "Form 470: Campaign Disclosure Statement, Short Form (Officeholders and Candidates)",
                    ),
                    (
                        "",
                        "Form 490: Campaign Disclosure Statement, Long Form (Officeholders and Candidates)",
                    ),
                    (
                        "",
                        "Form 495: Supplemental Pre-Election Campaign Statement (Recipient Committee)",
                    ),
                    ("", "Form 496: Late Independent Expenditure Report"),
                    ("", "Form 497: Late Contribution Report"),
                    ("", "Form 498: Late Payment Report (Slate Mailer Organization)"),
                    ("", "Form 501: Candidate Intention Statement"),
                    ("", "Form 502: Campaign Bank Account Statement"),
                    ("", "Form 511: Paid Spokesperson Report"),
                    ("", "Electronic Form 530: Electronic Issue Advocacy Report"),
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    ("", "Form 604: Lobbyist Certification Statement"),
                    (
                        "",
                        "Form 605: Amendment to Registration, Lobbying Firm, Lobbyist Employer, Lobbying Coalition",
                    ),
                    ("", "Form 606: Notice of Termination"),
                    ("", "Form 607: Notice of Withdrawal"),
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Schedule 630: Payments Made to Lobbying Coalitions (Attachment to Form 625 or 635)",
                    ),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    (
                        "S635-C",
                        "Schedule 635C: Payments Received by Lobbying Coalitions",
                    ),
                    (
                        "",
                        "Schedule 640: Governmental Agencies Reporting (Attachment to Form 635 or Form 645)",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                    ("", "Form 690: Amendment to Lobbying Disclosure Report"),
                    ("", "Form 700: Statement of Economic Interest"),
                    (
                        "",
                        "Form 900: Campaign Disclosure Statement (Public employee retirement board candidate)",
                    ),
                    ("F111", "Unknown"),
                    ("F410 AT", "Unknown"),
                    ("F410ATR", "Unknown"),
                    ("F421", "Unknown"),
                    ("F440", "Unknown"),
                    (
                        "F470S",
                        "Form 470: Campaign Disclosure Statement, Short Form (Officeholders and Candidates)",
                    ),
                    ("F480", "Unknown"),
                    ("F500", "Unknown"),
                    (
                        "F501502",
                        "Forms 501 and/or 502 (Candidate Intention and/or Bank Account Statements)",
                    ),
                    ("F555", "Unknown"),
                    ("F666", "Unknown"),
                    ("F777", "Unknown"),
                    ("F888", "Unknown"),
                    ("F999", "Unknown"),
                ],
                db_column="FORM_ID",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711614-CalAccessTablesWeb", start_page=65
                    )
                ],
                help_text="Form identification code",
                max_length=7,
                verbose_name="form type",
            ),
        ),
        migrations.AlterField(
            model_name="filerfilingscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerinterestscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerlinkscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filernamecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerstatustypescd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filertofilertypecd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filertypeperiodscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filerxrefcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filingperiodcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="filingscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="grouptypescd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="hdrcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="headercd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="imagelinkscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lattcd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Schedule 630: Payments Made to Lobbying Coalitions (Attachment to Form 625 or 635)",
                    ),
                    (
                        "S635-C",
                        "Schedule 635C: Payments Received by Lobbying Coalitions",
                    ),
                    (
                        "",
                        "Schedule 640: Governmental Agencies Reporting (Attachment to Form 635 or Form 645)",
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711616-MapCalFormat2Fields", start_page=52
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=65
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=79
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=6,
            ),
        ),
        migrations.AlterField(
            model_name="lattcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lccmcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="legislativesessionscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lempcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lexpcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="loancd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyamendmentscd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=74
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=88
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=9,
            ),
        ),
        migrations.AlterField(
            model_name="lobbyamendmentscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyingchglogcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistcontributions1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistcontributions2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistcontributions3cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemplobbyist1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemplobbyist2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployer1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployer2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployer3cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployerfirms1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployerfirms2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistemployerhistorycd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirm1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirm2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirm3cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmemployer1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmemployer2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmhistorycd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmlobbyist1cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lobbyistfirmlobbyist2cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lookupcodescd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lothcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="lpaycd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="namescd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="rawdatafile",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="rawdataversion",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="rcptcd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    ("", "Electronic Form 530: Electronic Issue Advocacy Report"),
                    (
                        "",
                        "Form 900: Campaign Disclosure Statement (Public employee retirement board candidate)",
                    ),
                    (
                        "F401A",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule A, Payments Received",
                    ),
                    (
                        "A",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule A, Monetary Contributions Received",
                    ),
                    (
                        "A-1",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule A-1, Contributions Transferred to Special Election Commitee",
                    ),
                    (
                        "C",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule C, Non-Monetary Contributions Received",
                    ),
                    (
                        "I",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule I, Miscellanous increases to cash",
                    ),
                    (
                        "F496P3",
                        "Form 496 (Late Independent Expenditure Report): Part 3, Contributions > $100 Received",
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=29
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=37
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=9,
            ),
        ),
        migrations.AlterField(
            model_name="rcptcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="form_id",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[
                    (
                        "",
                        "Form 400: Statement of Organization (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 401: Campaign Disclosure Statement (Slate Mailer Organization)",
                    ),
                    (
                        "",
                        "Form 402: Statement of Termination (Slate Mailer Organization)",
                    ),
                    ("", "Form 410: Statement of Organization (Recipient Committee)"),
                    (
                        "",
                        "Form 425: Semi-Annual Statement of No Activity (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("", "Form 465: Supplemental Independent Expenditure Report"),
                    ("", "Form 496: Late Independent Expenditure Report"),
                    ("", "Form 497: Late Contribution Report"),
                    ("", "Form 498: Late Payment Report (Slate Mailer Organization)"),
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    ("", "Form 604: Lobbyist Certification Statement"),
                    ("", "Form 606: Notice of Termination"),
                    ("", "Form 607: Notice of Withdrawal"),
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                ],
                db_column="FORM_ID",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=8, id="2711624-Overview", start_page=4
                    )
                ],
                help_text="Form identification code",
                max_length=7,
                verbose_name="form identification code",
            ),
        ),
        migrations.AlterField(
            model_name="receivedfilingscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="reportscd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="s401cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="s496cd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                blank=True,
                choices=[("", "Form 496: Late Independent Expenditure Report")],
                db_column="FORM_TYPE",
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=40
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=53
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="s496cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="s497cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="s498cd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="smrycd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 401: Campaign Disclosure Statement (Slate Mailer Organization)",
                    ),
                    (
                        "F401A",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule A, Payments Received",
                    ),
                    (
                        "F401B",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule B, Payments Made",
                    ),
                    (
                        "F401B-1",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule B-1, Payments Made by Agent or Independent Contractor",
                    ),
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "A",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule A, Monetary Contributions Received",
                    ),
                    (
                        "B1",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule B, Part 1, Loans Received",
                    ),
                    (
                        "B2",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule B, Part 2, Loan Guarantors",
                    ),
                    (
                        "B3",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule B, Part 3, Outstanding Balance",
                    ),
                    (
                        "C",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule C, Non-Monetary Contributions Received",
                    ),
                    (
                        "D",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule D, Summary of Expenditures Supporting / Opposing Other Candidates, Measures and Committees",
                    ),
                    (
                        "E",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule E, Payments Made",
                    ),
                    (
                        "F",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule F, Accrued Expenses (Unpaid Bills)",
                    ),
                    (
                        "G",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule G, Payments Made by an Agent or Independent Contractor (on Behalf of This Committee)",
                    ),
                    (
                        "H",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule H, Loans Made to Others",
                    ),
                    (
                        "H1",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule H, Part 1, Loans Made",
                    ),
                    (
                        "H2",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule H, Part 2, Repayments Rcvd",
                    ),
                    (
                        "H3",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule H, Part 3, Outstanding Loans",
                    ),
                    (
                        "I",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule I, Miscellanous increases to cash",
                    ),
                    (
                        "",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("", "Form 465: Supplemental Independent Expenditure Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "F625P2",
                        "Form 625 (Report of Lobbying Firm): Part 2, Payments Received in Connection with Lobbying Activity",
                    ),
                    (
                        "F625P3A",
                        "Form 625 (Report of Lobbying Firm): Part 3, Payments Made In Connection With Lobbying Activities, Section A, Activity Expenses",
                    ),
                    (
                        "F625P3B",
                        "Form 625 (Report of Lobbying Firm): Part 3, Payments Made In Connection With Lobbying Activities, Section B, Payments Made",
                    ),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    (
                        "F635P3A",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section A, Payments To In-house Employee Lobbyists",
                    ),
                    (
                        "F635P3B",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section B, Payments To Lobbying Firms",
                    ),
                    (
                        "F635P3C",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section C, Activity Expenses",
                    ),
                    (
                        "F635P3D",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section D, Other Payments to Influence Legislative or Administrative Action",
                    ),
                    (
                        "F635P3E",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section E, Payments in Connection with Administrative Testimony in Ratemaking Proceedings Before The California Public Utilities Commission",
                    ),
                    (
                        "",
                        "Schedule 640: Governmental Agencies Reporting (Attachment to Form 635 or Form 645)",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                    (
                        "F645P2A",
                        "Form 645 (Report of Person Spending $5,000 or More): Part 2, Payments Made this Period, Section A, Activity Expenses",
                    ),
                    (
                        "F645P2B",
                        "Form 645 (Report of Person Spending $5,000 or More): Part 2, Payments Made this Period, Section B, Other Payments to Influence Legislative or Administrative Action",
                    ),
                    (
                        "F645P2C",
                        "Form 645 (Report of Person Spending $5,000 or More): Part 2, Payments Made this Period, Section C, Payments in Connection with Administrative Testimony in Ratemaking Proceedings Before the California Public Utilities Commission",
                    ),
                    (
                        "",
                        "Form 900: Campaign Disclosure Statement (Public employee retirement board candidate)",
                    ),
                    (
                        "401A",
                        calaccess_raw.annotations.forms.FilingFormSection(
                            db_value="F401A",
                            documentcloud_id=None,
                            end_page="7",
                            form=calaccess_raw.annotations.forms.FilingForm(
                                db_value="",
                                description="Form 401 is filed by slate mailer organizations to disclose payments made and received in connection with producing slate mailers.",
                                documentcloud_id="2781366-401-2005-01",
                                group="CAMPAIGN",
                                id="F401",
                                title="Campaign Disclosure Statement (Slate Mailer Organization)",
                            ),
                            id="A",
                            start_page="5",
                            title="Schedule A, Payments Received",
                        ),
                    ),
                    (
                        "401B",
                        calaccess_raw.annotations.forms.FilingFormSection(
                            db_value="F401B",
                            documentcloud_id=None,
                            end_page="9",
                            form=calaccess_raw.annotations.forms.FilingForm(
                                db_value="",
                                description="Form 401 is filed by slate mailer organizations to disclose payments made and received in connection with producing slate mailers.",
                                documentcloud_id="2781366-401-2005-01",
                                group="CAMPAIGN",
                                id="F401",
                                title="Campaign Disclosure Statement (Slate Mailer Organization)",
                            ),
                            id="B",
                            start_page="8",
                            title="Schedule B, Payments Made",
                        ),
                    ),
                    (
                        "401B-1",
                        calaccess_raw.annotations.forms.FilingFormSection(
                            db_value="F401B-1",
                            documentcloud_id=None,
                            end_page="",
                            form=calaccess_raw.annotations.forms.FilingForm(
                                db_value="",
                                description="Form 401 is filed by slate mailer organizations to disclose payments made and received in connection with producing slate mailers.",
                                documentcloud_id="2781366-401-2005-01",
                                group="CAMPAIGN",
                                id="F401",
                                title="Campaign Disclosure Statement (Slate Mailer Organization)",
                            ),
                            id="B-1",
                            start_page="10",
                            title="Schedule B-1, Payments Made by Agent or Independent Contractor",
                        ),
                    ),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711616-MapCalFormat2Fields", start_page=86
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=28, id="2712033-Cal-Format-1-05-02", start_page=27
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=60, id="2712033-Cal-Format-1-05-02", start_page=59
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=37, id="2712034-Cal-Format-201", start_page=36
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        end_page=74, id="2712034-Cal-Format-201", start_page=73
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=8,
            ),
        ),
        migrations.AlterField(
            model_name="smrycd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="spltcd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="textmemocd",
            name="form_type",
            field=calaccess_raw.fields.CharField(
                choices=[
                    (
                        "",
                        "Form 401: Campaign Disclosure Statement (Slate Mailer Organization)",
                    ),
                    ("", "Form 405: Amendment to Campaign Disclosure Statement"),
                    ("", "Form 410: Statement of Organization (Recipient Committee)"),
                    (
                        "",
                        "Form 425: Semi-Annual Statement of No Activity (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 450: Campaign Disclosure Statement, Short Form (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("", "Form 465: Supplemental Independent Expenditure Report"),
                    ("", "Form 496: Late Independent Expenditure Report"),
                    ("", "Form 497: Late Contribution Report"),
                    ("", "Form 498: Late Payment Report (Slate Mailer Organization)"),
                    ("", "Form 601: Lobbying Firm Registration Statement"),
                    ("", "Form 602: Lobbying Firm Activity Authorization"),
                    (
                        "",
                        "Form 603: Lobbyist Employer or Lobbying Coalition Registration Statement",
                    ),
                    ("", "Form 604: Lobbyist Certification Statement"),
                    (
                        "",
                        "Form 605: Amendment to Registration, Lobbying Firm, Lobbyist Employer, Lobbying Coalition",
                    ),
                    ("", "Form 606: Notice of Termination"),
                    ("", "Form 607: Notice of Withdrawal"),
                    ("", "Form 615: Lobbyist Report"),
                    ("", "Form 625: Report of Lobbying Firm"),
                    (
                        "",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    ("", "Form 645: Report of Person Spending $5,000 or More"),
                    (
                        "",
                        "Schedule 630: Payments Made to Lobbying Coalitions (Attachment to Form 625 or 635)",
                    ),
                    (
                        "S635-C",
                        "Schedule 635C: Payments Received by Lobbying Coalitions",
                    ),
                    (
                        "",
                        "Schedule 640: Governmental Agencies Reporting (Attachment to Form 635 or Form 645)",
                    ),
                    (
                        "410",
                        "Form 410: Statement of Organization (Recipient Committee)",
                    ),
                    (
                        "460",
                        "Form 460: Campaign Disclosure Statement (Recipient Committee)",
                    ),
                    (
                        "461",
                        "Form 461: Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)",
                    ),
                    ("465", "Form 465: Supplemental Independent Expenditure Report"),
                    ("496", "Form 496: Late Independent Expenditure Report"),
                    ("497", "Form 497: Late Contribution Report"),
                    (
                        "497P1",
                        "Form 497 (Late Contribution Report): Part 1, Contributions Received",
                    ),
                    (
                        "497P2",
                        "Form 497 (Late Contribution Report): Part 2, Contributions Made",
                    ),
                    (
                        "F401A",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule A, Payments Received",
                    ),
                    (
                        "F401B",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule B, Payments Made",
                    ),
                    (
                        "F401B-1",
                        "Form 401 (Campaign Disclosure Statement (Slate Mailer Organization)): Schedule B-1, Payments Made by Agent or Independent Contractor",
                    ),
                    (
                        "F450P5",
                        "Form 450 (Campaign Disclosure Statement, Short Form (Recipient Committee)): Part 5, Payments Made",
                    ),
                    (
                        "F461P1",
                        "Form 461 (Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)): Part 1, Name and Address of Filer",
                    ),
                    (
                        "F461P2",
                        "Form 461 (Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)): Part 2, Nature and Interests of Filer",
                    ),
                    (
                        "F461P5",
                        "Form 461 (Campaign Disclosure Statement (Independent Expenditure Committee & Major Donor Committee)): Part 5, Contributions (Including Loans, Forgiveness of Loans, and LoanGuarantees) and Expenditures Made",
                    ),
                    (
                        "F465P3",
                        "Form 465 (Supplemental Independent Expenditure Report): Part 3, Independent Expenditures Made",
                    ),
                    (
                        "F496P3",
                        "Form 496 (Late Independent Expenditure Report): Part 3, Contributions > $100 Received",
                    ),
                    (
                        "F497P1",
                        "Form 497 (Late Contribution Report): Part 1, Contributions Received",
                    ),
                    (
                        "F497P2",
                        "Form 497 (Late Contribution Report): Part 2, Contributions Made",
                    ),
                    (
                        "F498-A",
                        "Form 498 (Late Payment Report (Slate Mailer Organization)): Part A, Late Payments Attributed To",
                    ),
                    (
                        "F498-R",
                        "Form 498 (Late Payment Report (Slate Mailer Organization)): Part R, Late Payments Received From",
                    ),
                    (
                        "F601P2A",
                        "Form 601 (Lobbying Firm Registration Statement): Part 2, Section A, Lobbyist Employers",
                    ),
                    (
                        "F601P2B",
                        "Form 601 (Lobbying Firm Registration Statement): Part 2, Section B, Subcontracted Clients",
                    ),
                    (
                        "F615P1",
                        "Form 615 (Lobbyist Report): Part 1, Activity Expenses Paid, Incurred, Arranged or Provided by the Lobbyist",
                    ),
                    (
                        "F615P2",
                        "Form 615 (Lobbyist Report): Part 2, Campaign Contributions Made or Delivered",
                    ),
                    (
                        "F625P2",
                        "Form 625 (Report of Lobbying Firm): Part 2, Payments Received in Connection with Lobbying Activity",
                    ),
                    (
                        "F625P3A",
                        "Form 625 (Report of Lobbying Firm): Part 3, Payments Made In Connection With Lobbying Activities, Section A, Activity Expenses",
                    ),
                    (
                        "F625P3B",
                        "Form 625 (Report of Lobbying Firm): Part 3, Payments Made In Connection With Lobbying Activities, Section B, Payments Made",
                    ),
                    (
                        "F625P4B",
                        "Form 625 (Report of Lobbying Firm): Part 4, Campaign Contributions Made",
                    ),
                    (
                        "S635",
                        "Form 635: Report of Lobbyist Employer or Report of Lobbying Coalition",
                    ),
                    (
                        "F635P3B",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section B, Payments To Lobbying Firms",
                    ),
                    (
                        "F635P3C",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 3, Payments Made in Connection with Lobbying Activities, Section C, Activity Expenses",
                    ),
                    (
                        "F635P4B",
                        "Form 635 (Report of Lobbyist Employer or Report of Lobbying Coalition): Part 4, Campaign Contributions Made",
                    ),
                    (
                        "F645P2A",
                        "Form 645 (Report of Person Spending $5,000 or More): Part 2, Payments Made this Period, Section A, Activity Expenses",
                    ),
                    (
                        "F645P3B",
                        "Form 645 (Report of Person Spending $5,000 or More): Part 3, Campaign Contributions Made",
                    ),
                    ("S497", "Form 497: Late Contribution Report"),
                    (
                        "S635C",
                        "Schedule 635C: Payments Received by Lobbying Coalitions",
                    ),
                    ("A", "Schedule A of any form (e.g., Forms 401 or 460)"),
                    ("A4", "Schedule A of any form (e.g., Forms 401 or 460)"),
                    ("A6", "Schedule A of any form (e.g., Forms 401 or 460)"),
                    ("B", "Schedule B of any form (e.g., Forms 401 or 460)"),
                    ("B1", "Schedule B, Part 1 of Forms 401 or 460"),
                    ("B2", "Schedule B, Part 2 of Forms 401 or 460"),
                    ("B3", "Schedule B, Part 3 of Forms 401 or 460"),
                    ("C", "Schedule C of any form (e.g., Forms 401 or F460)"),
                    ("COMMENTS", "Possibly comments by FPPC for any form?"),
                    ("CVR", "Cover page for any form (e.g., Forms 460, 461 or 497)"),
                    ("D", "Schedule D of any form (e.g., Forms 401, 460 or 461)"),
                    (
                        "DEBTF",
                        "Form 460 (Campaign Disclosure Statement (Recipient Committee)): Schedule F, Accrued Expenses (Unpaid Bills)",
                    ),
                    ("E", "Schedule E of any form (e.g., Forms 460, 461 or 465)"),
                    ("EXPNT", "Expenditures outlined on any form (e.g. Form 460)"),
                    ("F", "Schedule F of any form (e.g., Form 460)"),
                    ("G", "Schedule G of any form (e.g., Form 460)"),
                    ("H", "Schedule H of any form (e.g., Form 460)"),
                    ("H1", "Schedule H, Part 1 of any form (e.g., Form 460)"),
                    ("H2", "Schedule H2, Part 2 of any form (e.g., Form 460)"),
                    ("H3", "Schedule H3, Part 3 of any form (e.g., Form 460)"),
                    ("I", "Schedule I of any form (e.g., Form 460)"),
                    ("PT5", "Part 5 of any form (e.g., Form 461"),
                    ("RCPTB1", "Schedule B, Part 1 of any form (e.g., Form 460"),
                    ("RCPTC", "Schedule C of any form (e.g., Form 460)"),
                    ("RCPTI", "Schedule I of any form (e.g., Form 460)"),
                    ("SCH A", "Schedule A of any form (e.g., Form 460)"),
                    ("SF", "Schedule F of any form (e.g., Form 460)"),
                    ("SPLT", "A memo that applies to multiple items?"),
                    ("SMRY", "Summary section of any form (e.g., Form 460)"),
                    ("SUM", "Summary section of any form (e.g., Form 460)"),
                    ("SUMMARY", "Summary section of any form (e.g., Form 460)"),
                ],
                db_column="FORM_TYPE",
                db_index=True,
                documentcloud_pages=[
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2711616-MapCalFormat2Fields", start_page=90
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712034-Cal-Format-201", start_page=16
                    ),
                    calaccess_raw.annotations.documents.DocumentCloud(
                        id="2712033-Cal-Format-1-05-02", start_page=13
                    ),
                ],
                help_text="Name of the source filing form or schedule",
                max_length=8,
                verbose_name="form type",
            ),
        ),
        migrations.AlterField(
            model_name="textmemocd",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



"""Test commands."""
# Files
import io
import os

# Testing
import requests_mock
from django.test import TransactionTestCase
from django.test.utils import override_settings

# Django etc.
from django.conf import settings
from calaccess_raw import get_model_list
from calaccess_raw.management.commands.loadcalaccessrawfile import (
    Command as LoadCommand,
)
from calaccess_raw.management.commands.cleancalaccessrawfile import (
    Command as CleanCommand,
)
from calaccess_raw.management.commands.extractcalaccessrawfiles import (
    Command as ExtractCommand,
)
from calaccess_raw.management.commands.downloadcalaccessrawdata import (
    Command as DownloadCommand,
)

# Logging
import logging

logger = logging.getLogger(__name__)


@override_settings(
    CALACCESS_DATA_DIR=os.path.join(settings.BASE_DIR, "test-data"),
    MEDIA_ROOT=os.path.join(settings.BASE_DIR, "test-data", ".media"),
)
class CommandTestCase(TransactionTestCase):
    """
    Tests the management commands that interact with the database.
    """

    multi_db = True
    test_archiving = False

    @classmethod
    def setUpClass(cls):
        """
        Load data into the database before running other tests.
        """
        super(CommandTestCase, cls).setUpClass()
        with requests_mock.Mocker() as m:
            test_zip_path = os.path.join(
                settings.BASE_DIR,
                settings.CALACCESS_DATA_DIR,
                "dbwebexport.zip",
            )
            headers = {
                "Content-Length": str(os.stat(test_zip_path).st_size),
                "Accept-Ranges": "bytes",
                "Last-Modified": "Mon, 11 Jul 2017 11:20:31 GMT",
                "Connection": "keep-alive",
                "Date": "Mon, 10 Jul 2017 21:25:40 GMT",
                "Content-Type": "application/zip",
                "ETag": "2320c8-30619331-c54f7dc0",
                "Server": "Apache/2.2.3 (Red Hat)",
            }
            m.register_uri(
                "HEAD",
                "https://campaignfinance.cdn.sos.ca.gov/dbwebexport.zip",
                headers=headers,
            )
            m.register_uri(
                "GET",
                "https://campaignfinance.cdn.sos.ca.gov/dbwebexport.zip",
                headers=headers,
                content=io.open(test_zip_path, mode="rb").read(),
            )
            kwargs = dict(verbosity=3)
            dcmd = DownloadCommand()
            dcmd.handle(**kwargs)

        # Now archive the download
        suffix = "-test"  # f"-test-{get_random_string()}"
        print(f"Suffix: {suffix}")

        # Extract the data
        ecmd = ExtractCommand()
        ecmd.handle(verbosity=3, keep_files=True)

        # Clean the data
        tsv_list = [f for f in os.listdir(ecmd.tsv_dir) if ".TSV" in f.upper()]
        for i, name in enumerate(tsv_list):
            ccmd = CleanCommand()
            ccmd.handle(file_name=name, verbosity=3, keep_file=True)

        model_list = [
            x for x in get_model_list() if os.path.exists(x.objects.get_csv_path())
        ]
        for model in model_list:
            lcmd = LoadCommand()
            lcmd.handle(
                model_name=model.__name__,
                csv=model.objects.get_csv_path(),
                verbosity=3,
                keep_file=True,
            )

    def test_csv_gettrs(self):
        """
        Verify that get_csv_name methods work for all models.
        """
        for m in get_model_list():
            self.assertEqual(m.objects.get_csv_name(), m().get_csv_name())
            self.assertEqual(m.objects.get_csv_path(), m().get_csv_path())



"""
Tests the adminstration panel configuration.
"""
# Testing
from django.test import TestCase

# Stuff to test
from calaccess_raw import admin
from calaccess_raw import models
from calaccess_raw import get_model_list

# Logging
import logging

logger = logging.getLogger(__name__)


class AdminTestCase(TestCase):
    """
    Tests the adminstration panel configuration.
    """

    def test_models(self):
        """
        Make sure all the models have admins.
        """
        model_list = [m.__name__ for m in get_model_list()]
        admin_list = [a.replace("Admin", "") for a in admin.__all__]
        missing = set(model_list).difference(admin_list)
        self.assertEqual(missing, set([]))

    def test_methods(self):
        """
        Make sure our custom methods work.
        """
        a = admin.CvrSoCdAdmin(models.CvrSoCd(), None)
        a.get_readonly_fields()
        a.get_list_filter({})
        a.get_search_fields({})



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-26 16:43
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0009_auto_20161026_1641"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cvr2campaigndisclosurecd",
            name="cmte_id",
            field=calaccess_raw.fields.CharField(
                blank=True,
                db_column="CMTE_ID",
                help_text="Committee identification number, when the entity is a committee",
                max_length=9,
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-22 21:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0023_alter_rawdataversion_clean_zip_archive"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="rawdatafile",
            name="download_columns_count",
        ),
        migrations.RemoveField(
            model_name="rawdatafile",
            name="download_records_count",
        ),
        migrations.RemoveField(
            model_name="rawdatafile",
            name="error_count",
        ),
        migrations.RemoveField(
            model_name="rawdatafile",
            name="error_log_archive",
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the models from submodules and thread them together.
"""
from calaccess_raw.models.base import CalAccessBaseModel
from calaccess_raw.models.campaign import (
    CvrSoCd,
    Cvr2SoCd,
    CvrCampaignDisclosureCd,
    Cvr2CampaignDisclosureCd,
    Cvr3VerificationInfoCd,
    DebtCd,
    ExpnCd,
    LoanCd,
    RcptCd,
    S401Cd,
    F495P2Cd,
    S496Cd,
    S497Cd,
    S498Cd,
    F501502Cd,
)
from calaccess_raw.models.inactive import (
    BallotMeasuresCd,
    CvrF470Cd,
    FilerTypePeriodsCd,
    LobbyistContributions1Cd,
    LobbyistContributions2Cd,
    LobbyistContributions3Cd,
    LobbyistEmpLobbyist1Cd,
    LobbyistEmpLobbyist2Cd,
    LobbyistEmployer1Cd,
    LobbyistEmployer2Cd,
    LobbyistEmployer3Cd,
    LobbyistEmployerFirms1Cd,
    LobbyistEmployerFirms2Cd,
    LobbyistEmployerHistoryCd,
    LobbyistFirm1Cd,
    LobbyistFirm2Cd,
    LobbyistFirm3Cd,
    LobbyistFirmEmployer1Cd,
    LobbyistFirmEmployer2Cd,
    LobbyistFirmHistoryCd,
    LobbyistFirmLobbyist1Cd,
    LobbyistFirmLobbyist2Cd,
    EfsFilingLogCd,
)
from calaccess_raw.models.lobbying import (
    CvrRegistrationCd,
    Cvr2RegistrationCd,
    LobbyAmendmentsCd,
    LobbyingChgLogCd,
    LempCd,
    CvrLobbyDisclosureCd,
    Cvr2LobbyDisclosureCd,
    F690P2Cd,
    LattCd,
    LexpCd,
    LccmCd,
    LpayCd,
    LothCd,
)
from calaccess_raw.models.common import (
    CvrE530Cd,
    FilernameCd,
    FilerFilingsCd,
    FilingsCd,
    HdrCd,
    HeaderCd,
    SmryCd,
    SpltCd,
    TextMemoCd,
    AcronymsCd,
    AddressCd,
    FilersCd,
    FilerAcronymsCd,
    FilerAddressCd,
    FilerEthicsClassCd,
    FilerInterestsCd,
    FilerLinksCd,
    FilerStatusTypesCd,
    FilerToFilerTypeCd,
    FilerTypesCd,
    FilerXrefCd,
    FilingPeriodCd,
    GroupTypesCd,
    ImageLinksCd,
    LegislativeSessionsCd,
    LookupCodesCd,
    NamesCd,
    ReceivedFilingsCd,
    ReportsCd,
)

__all__ = (
    "CalAccessBaseModel",
    "CvrSoCd",
    "Cvr2SoCd",
    "CvrCampaignDisclosureCd",
    "Cvr2CampaignDisclosureCd",
    "CvrF470Cd",
    "RcptCd",
    "Cvr3VerificationInfoCd",
    "LoanCd",
    "S401Cd",
    "ExpnCd",
    "F495P2Cd",
    "DebtCd",
    "S496Cd",
    "SpltCd",
    "S497Cd",
    "F501502Cd",
    "S498Cd",
    "CvrRegistrationCd",
    "Cvr2RegistrationCd",
    "CvrLobbyDisclosureCd",
    "Cvr2LobbyDisclosureCd",
    "LobbyAmendmentsCd",
    "F690P2Cd",
    "LattCd",
    "LexpCd",
    "LccmCd",
    "LothCd",
    "LempCd",
    "LpayCd",
    "FilerFilingsCd",
    "FilingsCd",
    "SmryCd",
    "CvrE530Cd",
    "TextMemoCd",
    "AcronymsCd",
    "AddressCd",
    "BallotMeasuresCd",
    "EfsFilingLogCd",
    "FilernameCd",
    "FilersCd",
    "FilerAcronymsCd",
    "FilerAddressCd",
    "FilerEthicsClassCd",
    "FilerInterestsCd",
    "FilerLinksCd",
    "FilerStatusTypesCd",
    "FilerToFilerTypeCd",
    "FilerTypesCd",
    "FilerXrefCd",
    "FilingPeriodCd",
    "FilerTypePeriodsCd",
    "GroupTypesCd",
    "HeaderCd",
    "HdrCd",
    "ImageLinksCd",
    "LegislativeSessionsCd",
    "LobbyingChgLogCd",
    "LobbyistContributions1Cd",
    "LobbyistContributions2Cd",
    "LobbyistContributions3Cd",
    "LobbyistEmployer1Cd",
    "LobbyistEmployer2Cd",
    "LobbyistEmployer3Cd",
    "LobbyistEmployerHistoryCd",
    "LobbyistEmployerFirms1Cd",
    "LobbyistEmployerFirms2Cd",
    "LobbyistEmpLobbyist1Cd",
    "LobbyistEmpLobbyist2Cd",
    "LobbyistFirm1Cd",
    "LobbyistFirm2Cd",
    "LobbyistFirm3Cd",
    "LobbyistFirmEmployer1Cd",
    "LobbyistFirmEmployer2Cd",
    "LobbyistFirmLobbyist1Cd",
    "LobbyistFirmLobbyist2Cd",
    "LobbyistFirmHistoryCd",
    "LookupCodesCd",
    "NamesCd",
    "ReceivedFilingsCd",
    "ReportsCd",
)



"""
Extract the CAL-ACCESS raw data files from downloaded ZIP.
"""
import re
import os
import shutil
import zipfile

from calaccess_raw.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Extract the CAL-ACCESS raw data files from downloaded ZIP.
    """

    help = "Extract the CAL-ACCESS raw data files from downloaded ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep downloaded zipped files",
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header("Extracting raw data files")

        # flush tsv dir
        if os.path.exists(self.tsv_dir):
            shutil.rmtree(self.tsv_dir)
        os.mkdir(self.tsv_dir)

        # Do it
        self.extract_tsv_files()

        if not options["keep_files"]:
            shutil.rmtree(self.download_dir)

    def extract_tsv_files(self):
        """
        Extract all files with .TSV extension from downloaded zip.
        """
        if self.verbosity:
            self.log(" Extracting .TSV files")
        pattern = r"^.+\.TSV$"
        with zipfile.ZipFile(self.zip_path) as zf:
            tsv_files = [f for f in zf.namelist() if re.match(pattern, f)]
            for f in tsv_files:
                # extract
                extracted_path = zf.extract(f, self.download_dir)
                # move
                file_name = os.path.basename(extracted_path).upper()
                shutil.move(
                    extracted_path,
                    os.path.join(self.tsv_dir, file_name),
                )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-04 17:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("calaccess_raw", "0003_auto_20160804_1443"),
    ]

    operations = [
        migrations.AddField(
            model_name="rawdataversion",
            name="clean_zip_size",
            field=models.BigIntegerField(
                help_text="The size of the zip containing all cleaned raw data files and error logs.",
                null=True,
                verbose_name="clean zip size",
            ),
        ),
        migrations.AddField(
            model_name="rawdataversion",
            name="download_zip_size",
            field=models.BigIntegerField(
                help_text="The actual size of the downloaded CAL-ACCESS zip after the downloaded completed.",
                null=True,
                verbose_name="downloaded zip size",
            ),
        ),
        migrations.AlterField(
            model_name="rawdataversion",
            name="expected_size",
            field=models.BigIntegerField(
                help_text="The expected size of the downloaded CAL-ACCESS zip, as specified in the content-length field in HTTP response header.",
                verbose_name="expected downloaded zip size",
            ),
        ),
    ]



import os
from setuptools import setup


def read(file_name):
    """Read the provided file."""
    this_dir = os.path.dirname(__file__)
    file_path = os.path.join(this_dir, file_name)
    with open(file_path) as f:
        return f.read()


def version_scheme(version):
    """
    Version scheme hack for setuptools_scm.
    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342
    If that issue is resolved, this method can be removed.
    """
    import time

    from setuptools_scm.version import guess_next_version

    if version.exact:
        return version.format_with("{tag}")
    else:
        _super_value = version.format_next_version(guess_next_version)
        now = int(time.time())
        return _super_value + str(now)


def local_version(version):
    """
    Local version scheme hack for setuptools_scm.
    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342
    If that issue is resolved, this method can be removed.
    """
    return ""


setup(
    name='django-calaccess-raw-data',
    author='Ben Welsh',
    author_email='b@palewi.re',
    url='http://django-calaccess.californiacivicdata.org/',
    description=("A Django app to download, extract and load campaign "
                "finance and lobbying activity data from the California "
                "Secretary of State's CAL-ACCESS database"),
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    license='MIT',
    packages=(
        'calaccess_raw',
        'calaccess_raw.admin',
        'calaccess_raw.management',
        'calaccess_raw.migrations',
        'calaccess_raw.management.commands',
        'calaccess_raw.models',
        'calaccess_raw.annotations'
    ),
    include_package_data=True,
    zip_safe=False,
    install_requires=(
        'django>=4.0.*',
        'django-postgres-copy>=2.6.*',
        'requests',
        'csvkit',
        'django-internetarchive-storage',
    ),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: Django',
        'Framework :: Django :: 4',
        'License :: OSI Approved :: MIT License'
    ),
    project_urls={
        'Project': 'https://www.californiacivicdata.org/',
        'Documentation': 'https://django-calaccess.californiacivicdata.org/',
        'Funding': 'https://www.californiacivicdata.org/about/',
        'Source': 'https://github.com/palewire/django-calaccess-raw-data',
        'Testing': 'https://github.com/palewire/django-calaccess-raw-data/actions/workflows/tests.yaml',
        'Tracker': 'https://github.com/palewire/django-calaccess-raw-data/issues'
    },
    setup_requires=["setuptools_scm"],
    use_scm_version={"version_scheme": version_scheme, "local_scheme": local_version},
)



"""
Tests for the utilities storied in app's __init__.py file.
"""
# Testing
from django.test import TestCase
from django.test.utils import override_settings

# Stuff to test
import calaccess_raw

# Logging
import logging

logger = logging.getLogger(__name__)


class UtilityTestCase(TestCase):
    """
    Tests related to our hodgepodge of utilities.
    """

    multi_db = True

    @override_settings(CALACCESS_DATA_DIR=None)
    @override_settings(BASE_DIR=None)
    def test_dir_errors(self):
        """
        Test error expected when download directory is missing.
        """
        with self.assertRaises(ValueError):
            calaccess_raw.get_data_directory()

    @override_settings(CALACCESS_DATA_DIR="/foo/bar/")
    def test_dir_configured(self):
        """
        Tests for directory functions __init__.py file.
        """
        calaccess_raw.get_data_directory()

    @override_settings(CALACCESS_DATA_DIR=None)
    @override_settings(BASE_DIR="/foo/bar/")
    def test_dir_basedir(self):
        """
        Tests for directory functions __init__.py file with different settings.
        """
        calaccess_raw.get_data_directory()

    def test_model_methods(self):
        """
        Test the methods that hook up with our models.
        """
        calaccess_raw.get_model_list()



"""
A hodgepodge of utilities related to the app's settings and configuration.
"""
import os
from django.conf import settings

default_app_config = "calaccess_raw.apps.CalAccessRawConfig"


def get_data_directory():
    """
    Returns download directory for data storage downloaded data.
    """
    if getattr(settings, "CALACCESS_DATA_DIR", None):
        return getattr(settings, "CALACCESS_DATA_DIR")
    elif getattr(settings, "BASE_DIR", None):
        return os.path.join(getattr(settings, "BASE_DIR"), "data")
    raise ValueError(
        "CAL-ACCESS download directory not configured. Set either \
CALACCESS_DATA_DIR or BASE_DIR in settings.py"
    )


def get_model_list():
    """
    Returns a model list with all the data tables in this application.
    """
    from django.apps import apps

    model_list = apps.get_app_config("calaccess_raw").models.values()
    return [m for m in model_list if "CalAccessBaseModel" in str(m.__base__)]
